@@ -1,8 +1,8 @@
+import inspect
 from typing import (
     Any,
     Callable,
     Dict,
-    Iterable,
     Iterator,
     List,
     NamedTuple,
@@ -13,42 +13,21 @@ from typing import (
     cast,
 )
 from unittest import mock
+from warnings import warn
 
 import httpx
 
 from .patterns import M, Pattern
 from .types import (
-    ByteStream,
     CallableSideEffect,
+    Content,
     HeaderTypes,
-    RequestTypes,
-    Response,
+    ResolvedResponseTypes,
+    RouteResultTypes,
     SideEffectTypes,
 )
 
 
-def decode_request(request: RequestTypes) -> httpx.Request:
-    """
-    Build a httpx Request from httpcore request args.
-    """
-    if isinstance(request, httpx.Request):
-        return request  # pragma: nocover
-    method, url, headers, stream = request
-    return httpx.Request(method, url, headers=headers, stream=stream)
-
-
-def encode_response(response: httpx.Response) -> Response:
-    """
-    Builds a raw response tuple from httpx Response.
-    """
-    return (
-        response.status_code,
-        response.headers.raw,
-        response.stream,
-        response.extensions,
-    )
-
-
 def clone_response(response: httpx.Response, request: httpx.Request) -> httpx.Response:
     """
     Clones a httpx Response for given request.
@@ -60,8 +39,6 @@ def clone_response(response: httpx.Response, request: httpx.Request) -> httpx.Re
         request=request,
         extensions=dict(response.extensions),
     )
-    if isinstance(response.stream, Iterable):
-        response.read()  # Pre-read stream for easier call stats usage
     return response
 
 
@@ -96,7 +73,7 @@ class MockResponse(httpx.Response):
         self,
         status_code: Optional[int] = None,
         *,
-        content: Optional[Union[str, bytes, ByteStream]] = None,
+        content: Optional[Content] = None,
         content_type: Optional[str] = None,
         http_version: Optional[str] = None,
         **kwargs: Any,
@@ -255,11 +232,11 @@ class Route:
         status_code: int = 200,
         *,
         headers: Optional[HeaderTypes] = None,
-        content: Optional[Union[str, bytes, ByteStream]] = None,
+        content: Optional[Content] = None,
         text: Optional[str] = None,
         html: Optional[str] = None,
         json: Optional[Union[str, List, Dict]] = None,
-        stream: Optional[ByteStream] = None,
+        stream: Optional[Union[httpx.SyncByteStream, httpx.AsyncByteStream]] = None,
         content_type: Optional[str] = None,
         http_version: Optional[str] = None,
         **kwargs: Any,
@@ -309,15 +286,26 @@ class Route:
 
     def _call_side_effect(
         self, effect: CallableSideEffect, request: httpx.Request, **kwargs: Any
-    ) -> Optional[Union[httpx.Request, httpx.Response]]:
+    ) -> RouteResultTypes:
+        # Add route kwarg if the side effect wants it
+        argspec = inspect.getfullargspec(effect)
+        if "route" in kwargs:
+            warn(f"Matched context contains reserved word `route`: {self.pattern!r}")
+        if "route" in argspec.args:
+            kwargs["route"] = self
+
         try:
             # Call side effect
-            result = effect(request, **kwargs)
+            result: RouteResultTypes = effect(request, **kwargs)
         except Exception as error:
             raise SideEffectError(self, origin=error) from error
 
         # Validate result
-        if result and not isinstance(result, (httpx.Response, httpx.Request)):
+        if (
+            result
+            and not inspect.isawaitable(result)
+            and not isinstance(result, (httpx.Response, httpx.Request))
+        ):
             raise TypeError(
                 f"Side effects must return; either a `httpx.Response`,"
                 f"a `httpx.Request` for pass-through, "
@@ -328,7 +316,7 @@ class Route:
 
     def _resolve_side_effect(
         self, request: httpx.Request, **kwargs: Any
-    ) -> Optional[Union[httpx.Request, httpx.Response]]:
+    ) -> RouteResultTypes:
         effect = self._next_side_effect()
 
         # Handle Exception `instance` side effect
@@ -352,12 +340,11 @@ class Route:
             result = self._call_side_effect(effect, request, **kwargs)
             return result
 
+        # Resolved effect is a mocked response
         return effect
 
-    def resolve(
-        self, request: httpx.Request, **kwargs: Any
-    ) -> Optional[Union[httpx.Request, httpx.Response]]:
-        result: Optional[Union[httpx.Response, httpx.Request]] = None
+    def resolve(self, request: httpx.Request, **kwargs: Any) -> RouteResultTypes:
+        result: RouteResultTypes = None
 
         if self._side_effect:
             result = self._resolve_side_effect(request, **kwargs)
@@ -377,9 +364,7 @@ class Route:
 
         return result
 
-    def match(
-        self, request: httpx.Request
-    ) -> Optional[Union[httpx.Request, httpx.Response]]:
+    def match(self, request: httpx.Request) -> RouteResultTypes:
         """
         Matches and resolves request with given patterns and optional side effect.
 
@@ -498,6 +483,14 @@ class RouteList:
             return default
 
 
+class AllMockedAssertionError(AssertionError):
+    pass
+
+
+class AllCalledAssertionError(AssertionError):
+    pass
+
+
 class SideEffectError(Exception):
     def __init__(self, route: Route, origin: Exception) -> None:
         self.route = route
@@ -505,6 +498,13 @@ class SideEffectError(Exception):
 
 
 class PassThrough(Exception):
-    def __init__(self, message: str, *, request: httpx.Request) -> None:
+    def __init__(self, message: str, *, request: httpx.Request, origin: Route) -> None:
         super().__init__(message)
         self.request = request
+        self.origin = origin
+
+
+class ResolvedRoute:
+    def __init__(self):
+        self.route: Optional[Route] = None
+        self.response: Optional[ResolvedResponseTypes] = None
