@@ -5,6 +5,7 @@ from abc import ABC
 from enum import Enum
 from functools import reduce
 from http.cookies import SimpleCookie
+from json.decoder import JSONDecodeError
 from types import MappingProxyType
 from typing import (
     Any,
@@ -20,6 +21,7 @@ from typing import (
     Type,
     Union,
 )
+from unittest.mock import ANY
 from urllib.parse import urljoin
 
 import httpx
@@ -240,18 +242,37 @@ class MultiItemsMixin:
     lookup: Lookup
     value: Any
 
+    def _multi_items(
+        self, value: Any, *, parse_any: bool = False
+    ) -> Tuple[Tuple[str, Tuple[Any, ...]], ...]:
+        return tuple(
+            (
+                key,
+                tuple(
+                    ANY if parse_any and v == str(ANY) else v
+                    for v in value.get_list(key)
+                ),
+            )
+            for key in sorted(value.keys())
+        )
+
     def __hash__(self):
-        return hash((self.__class__, self.lookup, str(self.value)))
+        return hash((self.__class__, self.lookup, self._multi_items(self.value)))
+
+    def _eq(self, value: Any) -> Match:
+        value_items = self._multi_items(self.value, parse_any=True)
+        request_items = self._multi_items(value)
+        return Match(value_items == request_items)
 
     def _contains(self, value: Any) -> Match:
-        value_list = self.value.multi_items()
-        request_list = value.multi_items()
-
-        if len(value_list) > len(request_list):
+        if len(self.value.multi_items()) > len(value.multi_items()):
             return Match(False)
 
-        for item in value_list:
-            if item not in request_list:
+        value_items = self._multi_items(self.value, parse_any=True)
+        request_items = self._multi_items(value)
+
+        for item in value_items:
+            if item not in request_items:
                 return Match(False)
 
         return Match(True)
@@ -441,7 +462,10 @@ class JSON(ContentMixin, PathPattern):
 
     def parse(self, request: httpx.Request) -> str:
         content = super().parse(request)
-        json = jsonlib.loads(content.decode("utf-8"))
+        try:
+            json = jsonlib.loads(content.decode("utf-8"))
+        except JSONDecodeError:
+            return ""
 
         if self.path:
             value = json
