@@ -1,26 +1,37 @@
 import inspect
+from contextlib import contextmanager
 from functools import update_wrapper
 from types import TracebackType
 from typing import (
     Any,
     Callable,
     Dict,
+    Generator,
     List,
     NewType,
     Optional,
     Tuple,
     Type,
     Union,
+    cast,
     overload,
 )
-from warnings import warn
 
 import httpx
 
 from .mocks import Mocker
-from .models import CallList, PassThrough, Route, RouteList, SideEffectError
+from .models import (
+    AllCalledAssertionError,
+    AllMockedAssertionError,
+    CallList,
+    PassThrough,
+    ResolvedRoute,
+    Route,
+    RouteList,
+    SideEffectError,
+)
 from .patterns import Pattern, merge_patterns, parse_url_patterns
-from .types import DefaultType, URLPatternTypes
+from .types import DefaultType, ResolvedResponseTypes, RouteResultTypes, URLPatternTypes
 
 Default = NewType("Default", object)
 DEFAULT = Default(...)
@@ -88,9 +99,8 @@ class Router:
             route.reset()
 
     def assert_all_called(self) -> None:
-        assert all(
-            (route.called for route in self.routes)
-        ), "RESPX: some mocked requests were not called!"
+        if any(not route.called for route in self.routes):
+            raise AllCalledAssertionError("RESPX: some routes were not called!")
 
     def __getitem__(self, name: str) -> Route:
         return self.routes[name]
@@ -220,59 +230,87 @@ class Router:
         if route:
             route.calls.append(call)
 
-    def match(
-        self,
-        request: httpx.Request,
-    ) -> Tuple[Optional[Route], Optional[Union[httpx.Request, httpx.Response]]]:
-        route: Optional[Route] = None
-        response: Optional[Union[httpx.Request, httpx.Response]] = None
-
-        for prospect in self.routes:
-            response = prospect.match(request)
-            if response:
-                route = prospect
-                break
-
-        return route, response
-
-    def resolve(self, request: httpx.Request) -> Optional[httpx.Response]:
-        route: Optional[Route] = None
-        response: Optional[httpx.Response] = None
+    @contextmanager
+    def resolver(self, request: httpx.Request) -> Generator[ResolvedRoute, None, None]:
+        resolved = ResolvedRoute()
 
         try:
-            route, mock = self.match(request)
+            yield resolved
 
-            if route is None:
+            if resolved.route is None:
                 # Assert we always get a route match, if check is enabled
-                assert not self._assert_all_mocked, f"RESPX: {request!r} not mocked!"
+                if self._assert_all_mocked:
+                    raise AllMockedAssertionError(f"RESPX: {request!r} not mocked!")
 
                 # Auto mock a successful empty response
-                response = httpx.Response(200)
+                resolved.response = httpx.Response(200)
 
-            elif mock == request:
+            elif resolved.response == request:
                 # Pass-through request
-                response = None
+                raise PassThrough(
+                    f"Request marked to pass through: {request!r}",
+                    request=request,
+                    origin=resolved.route,
+                )
 
             else:
                 # Mocked response
-                assert isinstance(mock, httpx.Response)
-                response = mock
+                assert isinstance(resolved.response, httpx.Response)
 
         except SideEffectError as error:
             self.record(request, response=None, route=error.route)
-            raise error.origin
+            raise error.origin from error
+        except PassThrough:
+            self.record(request, response=None, route=resolved.route)
+            raise
         else:
-            self.record(request, response=response, route=route)
+            self.record(request, response=resolved.response, route=resolved.route)
 
-        return response
+    def resolve(self, request: httpx.Request) -> ResolvedRoute:
+        with self.resolver(request) as resolved:
+            for route in self.routes:
+                prospect = route.match(request)
+                if prospect is not None:
+                    resolved.route = route
+                    resolved.response = cast(ResolvedResponseTypes, prospect)
+                    break
+
+        if isinstance(resolved.response.stream, httpx.ByteStream):
+            resolved.response.read()  # Pre-read stream
+
+        return resolved
+
+    async def aresolve(self, request: httpx.Request) -> ResolvedRoute:
+        with self.resolver(request) as resolved:
+            for route in self.routes:
+                prospect: RouteResultTypes = route.match(request)
+
+                # Await async side effect and wrap any exception
+                if inspect.isawaitable(prospect):
+                    try:
+                        prospect = await prospect  # type: ignore
+                    except Exception as error:
+                        raise SideEffectError(route, origin=error) from error
+
+                if prospect is not None:
+                    resolved.route = route
+                    resolved.response = cast(ResolvedResponseTypes, prospect)
+                    break
+
+        if isinstance(resolved.response.stream, httpx.ByteStream):
+            await resolved.response.aread()  # Pre-read stream
+
+        return resolved
 
     def handler(self, request: httpx.Request) -> httpx.Response:
-        response = self.resolve(request)
-        if response is None:
-            raise PassThrough(
-                f"Request marked to pass through: {request!r}", request=request
-            )
-        return response
+        resolved = self.resolve(request)
+        assert isinstance(resolved.response, httpx.Response)
+        return resolved.response
+
+    async def async_handler(self, request: httpx.Request) -> httpx.Response:
+        resolved = await self.aresolve(request)
+        assert isinstance(resolved.response, httpx.Response)
+        return resolved.response
 
 
 class MockRouter(Router):
@@ -413,12 +451,3 @@ class MockRouter(Router):
                 self.reset()
             if self.Mocker:
                 self.Mocker.stop()
-
-
-class DeprecatedMockTransport(MockRouter):
-    def __init__(self, *args, **kwargs):
-        warn(
-            "MockTransport used as router is deprecated. Please use `respx.mock(...)`.",
-            category=DeprecationWarning,
-        )
-        super().__init__(*args, **kwargs)
